@@ -166,6 +166,8 @@ static void unlink_unix_socket(const struct sockaddr_storage *addr)
 
 int net_bound_socket(int type, const struct sockaddr_storage *addr, enum net_flags flags)
 {
+	int ret;
+
 	/* Create socket. */
 	int sock = net_unbound_socket(type, addr);
 	if (sock < 0) {
@@ -177,13 +179,6 @@ int net_bound_socket(int type, const struct sockaddr_storage *addr, enum net_fla
 		unlink_unix_socket(addr);
 	}
 
-	/* Reuse old address if taken. */
-	int ret = sockopt_enable(sock, SOL_SOCKET, SO_REUSEADDR);
-	if (ret != KNOT_EOK) {
-		close(sock);
-		return ret;
-	}
-
 	/* Don't bind IPv4 for IPv6 any address. */
 	if (addr->ss_family == AF_INET6) {
 		ret = sockopt_enable(sock, IPPROTO_IPV6, IPV6_V6ONLY);
@@ -457,6 +452,11 @@ static ssize_t io_exec(const struct io *io, int fd, struct msghdr *msg,
 					/* Timeouted, exit. */
 					return KNOT_ETIMEOUT;
 				} else {
+					/* In specific circumstances with Valgrind,
+					 * poll() returns wrong value.
+					 */
+					assert(ret <= 1);
+					assert(ret >= -1);
 					/* Other error, exit. */
 					return KNOT_ECONN;
 				}
