@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -116,7 +116,8 @@ static void init_cache(
 	conf_t *conf,
 	bool reinit_cache)
 {
-	/* For UDP, TCP and background workers, cache the numbers of running
+	/*
+	 * For UDP, TCP, XDP, and background workers, cache the number of running
 	 * workers. Cache the setting of TCP reuseport too. These values
 	 * can't change in runtime, while config data can.
 	 */
@@ -125,12 +126,14 @@ static void init_cache(
 	static bool   running_tcp_reuseport;
 	static size_t running_udp_threads;
 	static size_t running_tcp_threads;
+	static size_t running_xdp_threads;
 	static size_t running_bg_threads;
 
 	if (first_init || reinit_cache) {
 		running_tcp_reuseport = conf_tcp_reuseport(conf);
 		running_udp_threads = conf_udp_threads(conf);
 		running_tcp_threads = conf_tcp_threads(conf);
+		running_xdp_threads = conf_xdp_threads(conf);
 		running_bg_threads = conf_bg_threads(conf);
 
 		first_init = false;
@@ -184,6 +187,8 @@ static void init_cache(
 
 	conf->cache.srv_tcp_threads = running_tcp_threads;
 
+	conf->cache.srv_xdp_threads = running_xdp_threads;
+
 	conf->cache.srv_bg_threads = running_bg_threads;
 
 	conf->cache.srv_tcp_max_clients = conf_tcp_max_clients(conf);
@@ -375,6 +380,8 @@ int conf_clone(
 		out->hostname = strdup(s_conf->hostname);
 	}
 
+	out->catalog = s_conf->catalog;
+
 	// Initialize cached values.
 	init_cache(out, false);
 
