@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -73,6 +73,8 @@ typedef struct {
 dynarray_declare(mod, module_t *, DYNARRAY_VISIBILITY_PUBLIC, 16)
 dynarray_declare(old_schema, yp_item_t *, DYNARRAY_VISIBILITY_PUBLIC, 16)
 
+struct knot_catalog;
+
 /*! Configuration context. */
 typedef struct {
 	/*! Cloned configuration indicator. */
@@ -114,6 +116,7 @@ typedef struct {
 		bool srv_tcp_reuseport;
 		size_t srv_udp_threads;
 		size_t srv_tcp_threads;
+		size_t srv_xdp_threads;
 		size_t srv_bg_threads;
 		size_t srv_tcp_max_clients;
 		int ctl_timeout;
@@ -130,6 +133,8 @@ typedef struct {
 	list_t *query_modules;
 	/*! Default query modules plan. */
 	struct query_plan *query_plan;
+	/*! Zone catalog database. */
+	struct catalog *catalog;
 } conf_t;
 
 /*!
