@@ -24,13 +24,16 @@
 #include "knot/conf/confdb.h"
 #include "knot/common/log.h"
 #include "knot/server/dthreads.h"
+#include "knot/zone/catalog.h"
 #include "libknot/libknot.h"
 #include "libknot/yparser/yptrafo.h"
 #include "contrib/macros.h"
 #include "contrib/sockaddr.h"
+#include "contrib/strtonum.h"
 #include "contrib/string.h"
 #include "contrib/wire_ctx.h"
 #include "contrib/openbsd/strlcat.h"
+#include "contrib/openbsd/strlcpy.h"
 
 #define DBG_LOG(err) CONF_LOG(LOG_DEBUG, "%s (%s)", __func__, knot_strerror((err)));
 
@@ -177,6 +180,7 @@ conf_val_t conf_zone_get_txn(
 		CONF_LOG_ZONE(LOG_ERR, dname, "failed to read '%s/%s' (%s)",
 		              &C_ZONE[1], &key1_name[1], knot_strerror(val.code));
 		// FALLTHROUGH
+	case KNOT_YP_EINVAL_ID:
 	case KNOT_ENOENT:
 		break;
 	}
@@ -188,18 +192,40 @@ conf_val_t conf_zone_get_txn(
 		// Use the specified template.
 		conf_val(&val);
 		conf_db_get(conf, txn, C_TPL, key1_name, val.data, val.len, &val);
-		break;
+		goto got_template;
 	default:
 		CONF_LOG_ZONE(LOG_ERR, dname, "failed to read '%s/%s' (%s)",
 		              &C_ZONE[1], &C_TPL[1], knot_strerror(val.code));
 		// FALLTHROUGH
 	case KNOT_ENOENT:
 	case KNOT_YP_EINVAL_ID:
-		// Use the default template.
-		conf_db_get(conf, txn, C_TPL, key1_name, CONF_DEFAULT_ID + 1,
-		            CONF_DEFAULT_ID[0], &val);
+		break;
 	}
 
+	// Check if this is a catalog member zone.
+	if (conf->catalog != NULL) {
+		knot_dname_storage_t catalog;
+		int ret = catalog_get_zone_threadsafe(conf->catalog, dname, catalog);
+		if (ret == KNOT_EOK) {
+			conf_db_get(conf, txn, C_ZONE, C_CATALOG_TPL, catalog,
+			            knot_dname_size(catalog), &val);
+			if (val.code != KNOT_EOK) {
+				CONF_LOG_ZONE(LOG_ERR, catalog,
+				              "catalog zone has no catalog template (%s)",
+				              knot_strerror(val.code));
+				return val;
+			}
+			conf_val(&val);
+			conf_db_get(conf, txn, C_TPL, key1_name, val.data, val.len, &val);
+			goto got_template;
+		}
+	}
+
+	// Use the default template.
+	conf_db_get(conf, txn, C_TPL, key1_name, CONF_DEFAULT_ID + 1,
+	            CONF_DEFAULT_ID[0], &val);
+
+got_template:
 	switch (val.code) {
 	default:
 		CONF_LOG_ZONE(LOG_ERR, dname, "failed to read '%s/%s' (%s)",
@@ -650,6 +676,26 @@ struct sockaddr_storage conf_addr(
 	return out;
 }
 
+bool conf_addr_match(
+	conf_val_t *match,
+	const struct sockaddr_storage *addr)
+{
+	if (match == NULL || addr == NULL) {
+		return false;
+	}
+
+	while (match->code == KNOT_EOK) {
+		struct sockaddr_storage maddr = conf_addr(match, NULL);
+		if (sockaddr_cmp(&maddr, addr, true) == 0) {
+			return true;
+		}
+
+		conf_val_next(match);
+	}
+
+	return false;
+}
+
 struct sockaddr_storage conf_addr_range(
 	conf_val_t *val,
 	struct sockaddr_storage *max_ss,
@@ -791,6 +837,10 @@ conf_mod_id_t* conf_mod_id(
 void conf_free_mod_id(
 	conf_mod_id_t *mod_id)
 {
+	if (mod_id == NULL) {
+		return;
+	}
+
 	free(mod_id->name);
 	free(mod_id->data);
 	free(mod_id);
@@ -1057,6 +1107,15 @@ char* conf_zonefile_txn(
 	return get_filename(conf, txn, zone, file);
 }
 
+inline static bool legacy_db_fallback(
+	const yp_name_t *db_type)
+{
+	return (db_type[1] == C_JOURNAL_DB[1] ||
+	        db_type[1] == C_KASP_DB[1] ||
+	        db_type[1] == C_TIMER_DB[1]);
+	// warning comparing by first letter as every database has different!
+}
+
 char* conf_db_txn(
 	conf_t *conf,
 	knot_db_txn_t *txn,
@@ -1068,7 +1127,7 @@ char* conf_db_txn(
 	}
 
 	conf_val_t db_val = conf_get_txn(conf, txn, C_DB, db_type);
-	if (db_val.code != KNOT_EOK) {
+	if (db_val.code != KNOT_EOK && legacy_db_fallback(db_type)) {
 		db_val = conf_default_get_txn(conf, txn, db_type);
 	}
 
@@ -1086,7 +1145,7 @@ conf_val_t conf_db_param_txn(
 	const yp_name_t *legacy_param)
 {
 	conf_val_t val = conf_get_txn(conf, txn, C_DB, param);
-	if (val.code != KNOT_EOK) {
+	if (val.code != KNOT_EOK && legacy_param != NULL) {
 		val = conf_default_get_txn(conf, txn, legacy_param);
 	}
 
@@ -1127,6 +1186,26 @@ size_t conf_tcp_threads_txn(
 	return workers;
 }
 
+size_t conf_xdp_threads_txn(
+	conf_t *conf,
+	knot_db_txn_t *txn)
+{
+	size_t workers = 0;
+
+	conf_val_t val = conf_get_txn(conf, txn, C_SRV, C_LISTEN_XDP);
+	while (val.code == KNOT_EOK) {
+		struct sockaddr_storage addr = conf_addr(&val, NULL);
+		conf_xdp_iface_t iface;
+		int ret = conf_xdp_iface(&addr, &iface);
+		if (ret == KNOT_EOK) {
+			workers += iface.queues;
+		}
+		conf_val_next(&val);
+	}
+
+	return workers;
+}
+
 size_t conf_bg_threads_txn(
 	conf_t *conf,
 	knot_db_txn_t *txn)
@@ -1280,3 +1359,53 @@ conf_remote_t conf_remote_txn(
 
 	return out;
 }
+
+int conf_xdp_iface(
+	struct sockaddr_storage *addr,
+	conf_xdp_iface_t *iface)
+{
+#ifndef ENABLE_XDP
+	return KNOT_ENOTSUP;
+#else
+	if (addr == NULL || iface == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	if (addr->ss_family == AF_UNIX) {
+		const char *addr_str = ((struct sockaddr_un *)addr)->sun_path;
+		strlcpy(iface->name, addr_str, sizeof(iface->name));
+
+		const char *port = strchr(addr_str, '@');
+		if (port != NULL) {
+			iface->name[port - addr_str] = '\0';
+			int ret = str_to_u16(port + 1, &iface->port);
+			if (ret != KNOT_EOK) {
+				return ret;
+			} else if (iface->port == 0) {
+				return KNOT_EINVAL;
+			}
+		} else {
+			iface->port = 53;
+		}
+	} else {
+		int ret = knot_eth_name_from_addr(addr, iface->name, sizeof(iface->name));
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		ret = sockaddr_port(addr);
+		if (ret < 1) {
+			return KNOT_EINVAL;
+		}
+		iface->port = ret;
+	}
+
+	int queues = knot_eth_queues(iface->name);
+	if (queues <= 0) {
+		assert(queues != 0);
+		return queues;
+	}
+	iface->queues = queues;
+
+	return KNOT_EOK;
+#endif
+}
