@@ -22,6 +22,7 @@
 #define C_ACL			"\x03""acl"
 #define C_ACTION		"\x06""action"
 #define C_ADDR			"\x07""address"
+#define C_ADJUST_THR		"\x0E""adjust-threads"
 #define C_ALG			"\x09""algorithm"
 #define C_ANS_ROTATION		"\x0F""answer-rotation"
 #define C_ANY			"\x03""any"
@@ -30,6 +31,10 @@
 #define C_BACKEND		"\x07""backend"
 #define C_BG_WORKERS		"\x12""background-workers"
 #define C_BLOCK_NOTIFY_XFR	"\x1B""block-notify-after-transfer"
+#define C_CATALOG_DB            "\x0A""catalog-db"
+#define C_CATALOG_DB_MAX_SIZE   "\x13""catalog-db-max-size"
+#define C_CATALOG_ROLE		"\x0C""catalog-role"
+#define C_CATALOG_TPL		"\x10""catalog-template"
 #define C_CDS_CDNSKEY		"\x13""cds-cdnskey-publish"
 #define C_CHK_INTERVAL		"\x0E""check-interval"
 #define C_COMMENT		"\x07""comment"
@@ -42,6 +47,7 @@
 #define C_DNSKEY_TTL		"\x0A""dnskey-ttl"
 #define C_DNSSEC_POLICY		"\x0D""dnssec-policy"
 #define C_DNSSEC_SIGNING	"\x0E""dnssec-signing"
+#define C_DNSSEC_VALIDATION	"\x11""dnssec-validation"
 #define C_DOMAIN		"\x06""domain"
 #define C_DS_PUSH		"\x07""ds-push"
 #define C_ECS			"\x12""edns-client-subnet"
@@ -65,6 +71,7 @@
 #define C_KSK_SHARED		"\x0a""ksk-shared"
 #define C_KSK_SIZE		"\x08""ksk-size"
 #define C_LISTEN		"\x06""listen"
+#define C_LISTEN_XDP		"\x0A""listen-xdp"
 #define C_LOG			"\x03""log"
 #define C_MANUAL		"\x06""manual"
 #define C_MASTER		"\x06""master"
@@ -84,6 +91,7 @@
 #define C_PROPAG_DELAY		"\x11""propagation-delay"
 #define C_REFRESH_MAX_INTERVAL	"\x14""refresh-max-interval"
 #define C_REFRESH_MIN_INTERVAL	"\x14""refresh-min-interval"
+#define C_REPRO_SIGNING		"\x14""reproducible-signing"
 #define C_RMT			"\x06""remote"
 #define C_RRSIG_LIFETIME	"\x0E""rrsig-lifetime"
 #define C_RRSIG_PREREFRESH	"\x11""rrsig-pre-refresh"
@@ -183,6 +191,11 @@ enum {
 	ZONEFILE_LOAD_DIFSE = 3,
 };
 
+enum {
+	CATALOG_ROLE_NONE      = 0,
+	CATALOG_ROLE_INTERPRET = 1,
+};
+
 extern const knot_lookup_t acl_actions[];
 
 extern const yp_item_t conf_schema[];
