@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #include "knot/common/log.h"
 #include "libknot/errcode.h"
 #include "libknot/yparser/yptrafo.h"
+#include "contrib/sockaddr.h"
 #include "contrib/string.h"
 #include "contrib/wire_ctx.h"
 
@@ -160,6 +161,47 @@ int rrtype_to_txt(
 	YP_CHECK_RET;
 }
 
+int rdname_to_bin(
+	YP_TXT_BIN_PARAMS)
+{
+	YP_CHECK_PARAMS_BIN;
+
+	int ret = yp_dname_to_bin(in, out, stop);
+	if (ret == KNOT_EOK && in->wire[in->size - 1] != '.') {
+		// If non-FQDN, trim off the zero label.
+		wire_ctx_skip(out, -1);
+	}
+
+	YP_CHECK_RET;
+}
+
+int rdname_to_txt(
+	YP_BIN_TXT_PARAMS)
+{
+	YP_CHECK_PARAMS_TXT;
+
+	// Temporarily normalize the input.
+	if (in->wire[in->size - 1] == '\0') {
+		return yp_dname_to_txt(in, out);
+	}
+
+	knot_dname_storage_t full_name;
+	wire_ctx_t ctx = wire_ctx_init(full_name, sizeof(full_name));
+	wire_ctx_write(&ctx, in->wire, in->size);
+	wire_ctx_write(&ctx, "\0", 1);
+	wire_ctx_set_offset(&ctx, 0);
+
+	int ret = yp_dname_to_txt(&ctx, out);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	// Trim off the trailing dot.
+	wire_ctx_skip(out, -1);
+
+	YP_CHECK_RET;
+}
+
 int check_ref(
 	knotd_conf_check_args_t *args)
 {
@@ -186,6 +228,56 @@ int check_ref_dflt(
 	return KNOT_EOK;
 }
 
+int check_listen(
+	knotd_conf_check_args_t *args)
+{
+	bool no_port;
+	struct sockaddr_storage ss = yp_addr(args->data, &no_port);
+	if (!no_port && sockaddr_port(&ss) == 0) {
+		args->err_str = "invalid port";
+		return KNOT_EINVAL;
+	}
+
+	return KNOT_EOK;
+}
+
+int check_xdp(
+	knotd_conf_check_args_t *args)
+{
+#ifndef ENABLE_XDP
+		args->err_str = "XDP is not available";
+		return KNOT_ENOTSUP;
+#else
+	bool no_port;
+	struct sockaddr_storage ss = yp_addr(args->data, &no_port);
+	conf_xdp_iface_t if_new;
+	int ret = conf_xdp_iface(&ss, &if_new);
+	if (ret != KNOT_EOK) {
+		args->err_str = "invalid XDP interface specification";
+		return ret;
+	}
+
+	conf_val_t xdp = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
+	                              C_LISTEN_XDP);
+	size_t count = conf_val_count(&xdp);
+	while (xdp.code == KNOT_EOK && count-- > 1) {
+		struct sockaddr_storage addr = conf_addr(&xdp, NULL);
+		conf_xdp_iface_t if_prev;
+		ret = conf_xdp_iface(&addr, &if_prev);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		if (strcmp(if_new.name, if_prev.name) == 0) {
+			args->err_str = "duplicate XDP interface specification";
+			return KNOT_EINVAL;
+		}
+		conf_val_next(&xdp);
+	}
+
+	return KNOT_EOK;
+#endif
+}
+
 int check_modref(
 	knotd_conf_check_args_t *args)
 {
@@ -259,10 +351,18 @@ int check_module_id(
 int check_server(
 	knotd_conf_check_args_t *args)
 {
+	conf_val_t listen = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
+	                                 C_LISTEN);
+	conf_val_t xdp = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
+	                              C_LISTEN_XDP);
+	if (xdp.code == KNOT_EOK && listen.code != KNOT_EOK) {
+		CONF_LOG(LOG_WARNING, "unable to process TCP queries due to XDP-only interfaces");
+	}
+
 	conf_val_t hshake = conf_get_txn(args->extra->conf, args->extra->txn, C_SRV,
 	                                 C_TCP_HSHAKE_TIMEOUT);
 	if (hshake.code == KNOT_EOK) {
-		CONF_LOG(LOG_NOTICE, "option 'tcp-handshake-timeout' is no longer supported");
+		CONF_LOG(LOG_NOTICE, "option 'server.tcp-handshake-timeout' is no longer supported");
 	}
 
 	CHECK_LEGACY_NAME(C_SRV, C_TCP_REPLY_TIMEOUT, C_TCP_RMT_IO_TIMEOUT);
@@ -420,6 +520,18 @@ int check_acl(
 		return KNOT_EINVAL;
 	}
 
+	conf_val_t addr = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_ACL,
+	                                     C_ADDR, args->id, args->id_len);
+	conf_val_t key = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_ACL,
+	                                    C_KEY, args->id, args->id_len);
+	conf_val_t remote = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_ACL,
+	                                       C_RMT, args->id, args->id_len);
+	if (remote.code != KNOT_ENOENT &&
+	    (addr.code != KNOT_ENOENT || key.code != KNOT_ENOENT)) {
+		args->err_str = "specified ACL/remote together with address or key";
+		return KNOT_EINVAL;
+	}
+
 	return KNOT_EOK;
 }
 
@@ -503,6 +615,25 @@ int check_zone(
 		CONF_LOG(LOG_NOTICE, "option 'disable-any' is deprecated and has no effect");
 	}
 
+	conf_val_t signing = conf_zone_get_txn(args->extra->conf, args->extra->txn,
+	                                       C_DNSSEC_SIGNING, yp_dname(args->id));
+	conf_val_t validation = conf_zone_get_txn(args->extra->conf, args->extra->txn,
+	                                          C_DNSSEC_VALIDATION, yp_dname(args->id));
+	if (conf_bool(&signing) == true && conf_bool(&validation) == true) {
+		args->err_str = "'dnssec-validation' is not compatible with 'dnssec-signing'";
+		return KNOT_EINVAL;
+	}
+
+	conf_val_t catalog_role = conf_zone_get_txn(args->extra->conf, args->extra->txn,
+	                                            C_CATALOG_ROLE, yp_dname(args->id));
+	conf_val_t catalog_tpl = conf_zone_get_txn(args->extra->conf, args->extra->txn,
+	                                           C_CATALOG_TPL, yp_dname(args->id));
+	if ((bool)(conf_opt(&catalog_role) == CATALOG_ROLE_INTERPRET) !=
+	    (bool)(catalog_tpl.code == KNOT_EOK)) {
+		args->err_str = "'catalog-role' must correspond to configured 'catalog-template'";
+		return KNOT_EINVAL;
+	}
+
 	return KNOT_EOK;
 }
 
