@@ -29,6 +29,7 @@
 #include "knot/journal/journal_metadata.h"
 #include "knot/nameserver/query_module.h"
 #include "knot/updates/zone-update.h"
+#include "knot/zone/backup.h"
 #include "knot/zone/timers.h"
 #include "knot/zone/zonefile.h"
 #include "libknot/libknot.h"
@@ -146,14 +147,21 @@ static int get_zone(ctl_args_t *args, zone_t **zone)
 
 static int zones_apply(ctl_args_t *args, int (*fcn)(zone_t *, ctl_args_t *))
 {
+	int ret = KNOT_EOK;
+
 	// Process all configured zones if none is specified.
 	if (args->data[KNOT_CTL_IDX_ZONE] == NULL) {
+		args->failed = false;
 		knot_zonedb_foreach(args->server->zone_db, fcn, args);
+		if (args->failed) {
+			ret = KNOT_CTL_EZONE;
+			log_ctl_error("control, error (%s)", knot_strerror(ret));
+			send_error(args, knot_strerror(ret));
+			args->failed = false;
+		}
 		return KNOT_EOK;
 	}
 
-	int ret = KNOT_EOK;
-
 	while (true) {
 		zone_t *zone;
 		ret = get_zone(args, &zone);
@@ -365,6 +373,7 @@ static int zone_flush(zone_t *zone, ctl_args_t *args)
 		if (ret != KNOT_EOK) {
 			log_zone_warning(zone->name, "failed to update zone file (%s)",
 			                 knot_strerror(ret));
+			args->failed = true;
 		}
 		return ret;
 	}
@@ -378,6 +387,70 @@ static int zone_flush(zone_t *zone, ctl_args_t *args)
 	return KNOT_EOK;
 }
 
+static int init_backup(ctl_args_t *args, bool restore_mode)
+{
+	if (!MATCH_AND_FILTER(args, CTL_FILTER_FLUSH_OUTDIR)) {
+		return KNOT_EINVAL;
+	}
+
+	zone_backup_ctx_t *ctx;
+
+	// The present timer db size is not up-to-date, use the maximum one.
+	conf_val_t timer_db_size = conf_db_param(conf(), C_TIMER_DB_MAX_SIZE,
+	                                         C_MAX_TIMER_DB_SIZE);
+
+	int ret = zone_backup_init(restore_mode,
+	                           args->data[KNOT_CTL_IDX_DATA],
+	                           knot_lmdb_copy_size(&args->server->kaspdb),
+	                           conf_int(&timer_db_size),
+	                           knot_lmdb_copy_size(&args->server->journaldb),
+	                           knot_lmdb_copy_size(&args->server->catalog.db),
+	                           &ctx);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	assert(ctx != NULL);
+	ctx->backup_journal = MATCH_AND_FILTER(args, CTL_FILTER_PURGE_JOURNAL);
+	ctx->backup_zonefile = !MATCH_AND_FILTER(args, CTL_FILTER_PURGE_ZONEFILE);
+	args->custom_ctx = ctx;
+
+	if (args->data[KNOT_CTL_IDX_ZONE] == NULL) {
+		ctx->backup_global = true;
+		ret = global_backup(ctx, &args->server->catalog, NULL);
+	}
+
+	return ret;
+}
+
+static void deinit_backup(ctl_args_t *args)
+{
+	zone_backup_ctx_t *ctx = args->custom_ctx;
+	zone_backup_deinit(ctx);
+}
+
+static int zone_backup_cmd(zone_t *zone, ctl_args_t *args)
+{
+	zone_backup_ctx_t *ctx = args->custom_ctx;
+	if (zone->backup_ctx != NULL) {
+		log_zone_warning(zone->name, "back-up already in progress");
+		args->failed = true;
+		return KNOT_EPROGRESS;
+	}
+	zone->backup_ctx = ctx;
+	pthread_mutex_lock(&ctx->readers_mutex);
+	ctx->readers++;
+	pthread_mutex_unlock(&ctx->readers_mutex);
+
+	schedule_trigger(zone, args, ZONE_EVENT_BACKUP, true);
+
+	if (ctx->backup_global) {
+		return KNOT_EOK;
+	} else {
+		return global_backup(ctx, zone->catalog, zone->name);
+	}
+}
+
 static int zone_sign(zone_t *zone, ctl_args_t *args)
 {
 	UNUSED(args);
@@ -503,7 +576,7 @@ static int zone_txn_commit(zone_t *zone, ctl_args_t *args)
 		zone_sign_reschedule_t resch = { 0 };
 		bool full = (zone->control_update->flags & UPDATE_FULL);
 		zone_sign_roll_flags_t rflags = KEY_ROLL_ALLOW_ALL;
-		int ret = (full ? knot_dnssec_zone_sign(zone->control_update, 0, rflags, &resch) :
+		int ret = (full ? knot_dnssec_zone_sign(zone->control_update, 0, rflags, 0, &resch) :
 		                  knot_dnssec_sign_update(zone->control_update, &resch));
 		if (ret != KNOT_EOK) {
 			zone_control_clear(zone);
@@ -1146,13 +1219,14 @@ static int zone_purge(zone_t *zone, ctl_args_t *args)
 	return KNOT_EOK;
 }
 
-static int send_stats_ctr(mod_ctr_t *ctr, ctl_args_t *args, knot_ctl_data_t *data)
+static int send_stats_ctr(mod_ctr_t *ctr, uint64_t **stats_vals, unsigned threads,
+                          ctl_args_t *args, knot_ctl_data_t *data)
 {
 	char index[128];
 	char value[32];
 
 	if (ctr->count == 1) {
-		uint64_t counter = ATOMIC_GET(ctr->counter);
+		uint64_t counter = stats_get_counter(stats_vals, ctr->offset, threads);
 		int ret = snprintf(value, sizeof(value), "%"PRIu64, counter);
 		if (ret <= 0 || ret >= sizeof(value)) {
 			return KNOT_ESPACE;
@@ -1170,7 +1244,7 @@ static int send_stats_ctr(mod_ctr_t *ctr, ctl_args_t *args, knot_ctl_data_t *dat
 		                          CTL_FLAG_FORCE);
 
 		for (uint32_t i = 0; i < ctr->count; i++) {
-			uint64_t counter = ATOMIC_GET(ctr->counters[i]);
+			uint64_t counter = stats_get_counter(stats_vals, ctr->offset + i, threads);
 
 			// Skip empty counters.
 			if (counter == 0 && !force) {
@@ -1247,8 +1321,10 @@ static int modules_stats(list_t *query_modules, ctl_args_t *args, knot_dname_t *
 
 		data[KNOT_CTL_IDX_SECTION] = mod->id->name + 1;
 
+		unsigned threads = knotd_mod_threads(mod);
+
 		for (int i = 0; i < mod->stats_count; i++) {
-			mod_ctr_t *ctr = mod->stats + i;
+			mod_ctr_t *ctr = mod->stats_info + i;
 
 			// Skip empty counter.
 			if (ctr->name == NULL) {
@@ -1277,7 +1353,7 @@ static int modules_stats(list_t *query_modules, ctl_args_t *args, knot_dname_t *
 			data[KNOT_CTL_IDX_ITEM] = ctr->name;
 
 			// Send the counters.
-			int ret = send_stats_ctr(ctr, args, &data);
+			int ret = send_stats_ctr(ctr, mod->stats_vals, threads, args, &data);
 			if (ret != KNOT_EOK) {
 				return ret;
 			}
@@ -1294,6 +1370,7 @@ static int zone_stats(zone_t *zone, ctl_args_t *args)
 
 static int ctl_zone(ctl_args_t *args, ctl_cmd_t cmd)
 {
+	int ret;
 	switch (cmd) {
 	case CTL_ZONE_STATUS:
 		return zones_apply(args, zone_status);
@@ -1307,6 +1384,20 @@ static int ctl_zone(ctl_args_t *args, ctl_cmd_t cmd)
 		return zones_apply(args, zone_notify);
 	case CTL_ZONE_FLUSH:
 		return zones_apply(args, zone_flush);
+	case CTL_ZONE_BACKUP:
+		ret = init_backup(args, false);
+		if (ret == KNOT_EOK) {
+			ret = zones_apply(args, zone_backup_cmd);
+			deinit_backup(args);
+		}
+		return ret;
+	case CTL_ZONE_RESTORE:
+		ret = init_backup(args, true);
+		if (ret == KNOT_EOK) {
+			ret = zones_apply(args, zone_backup_cmd);
+			deinit_backup(args);
+		}
+		return ret;
 	case CTL_ZONE_SIGN:
 		return zones_apply(args, zone_sign);
 	case CTL_ZONE_KEY_ROLL:
@@ -1363,10 +1454,11 @@ static int server_status(ctl_args_t *args)
 	} else if (strcasecmp(type, "workers") == 0) {
 		int running_bkg_wrk, wrk_queue;
 		worker_pool_status(args->server->workers, &running_bkg_wrk, &wrk_queue);
-		ret = snprintf(buff, sizeof(buff), "UDP workers: %zu, TCP workers %zu, "
-		               "background workers: %zu (running: %d, pending: %d)",
+		ret = snprintf(buff, sizeof(buff), "UDP workers: %zu, TCP workers: %zu, "
+		               "XDP workers: %zu, background workers: %zu (running: %d, pending: %d)",
 		               conf()->cache.srv_udp_threads, conf()->cache.srv_tcp_threads,
-		               conf()->cache.srv_bg_threads, running_bkg_wrk, wrk_queue);
+		               conf()->cache.srv_xdp_threads, conf()->cache.srv_bg_threads,
+		               running_bkg_wrk, wrk_queue);
 	} else if (strcasecmp(type, "configure") == 0) {
 		ret = snprintf(buff, sizeof(buff), "%s", CONFIGURE_SUMMARY);
 	} else {
@@ -1748,6 +1840,8 @@ static const desc_t cmd_table[] = {
 	[CTL_ZONE_RETRANSFER] = { "zone-retransfer",    ctl_zone },
 	[CTL_ZONE_NOTIFY]     = { "zone-notify",        ctl_zone },
 	[CTL_ZONE_FLUSH]      = { "zone-flush",         ctl_zone },
+	[CTL_ZONE_BACKUP]     = { "zone-backup",        ctl_zone },
+	[CTL_ZONE_RESTORE]    = { "zone-restore",       ctl_zone },
 	[CTL_ZONE_SIGN]       = { "zone-sign",          ctl_zone },
 	[CTL_ZONE_KEY_ROLL]   = { "zone-key-rollover",  ctl_zone },
 	[CTL_ZONE_KSK_SBM]    = { "zone-ksk-submitted", ctl_zone },
