@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -81,6 +81,9 @@ static void policy_load(knot_kasp_policy_t *policy, conf_val_t *id)
 	val = conf_id_get(conf(), C_POLICY, C_RRSIG_PREREFRESH, id);
 	policy->rrsig_prerefresh = conf_int(&val);
 
+	val = conf_id_get(conf(), C_POLICY, C_REPRO_SIGNING, id);
+	policy->reproducible_sign = conf_bool(&val);
+
 	val = conf_id_get(conf(), C_POLICY, C_NSEC3, id);
 	policy->nsec3_enabled = conf_bool(&val);
 
@@ -170,6 +173,13 @@ int kdnssec_ctx_init(conf_t *conf, kdnssec_ctx_t *ctx, const knot_dname_t *zone_
 		goto init_error;
 	}
 
+	ret = kasp_db_get_saved_ttls(ctx->kasp_db, zone_name,
+	                             &ctx->policy->saved_max_ttl,
+	                             &ctx->policy->saved_key_ttl);
+	if (ret != KNOT_EOK && ret != KNOT_ENOENT) {
+		return ret;
+	}
+
 	conf_val_t policy_id;
 	if (from_module == NULL) {
 		policy_id = conf_zone_get(conf, C_DNSSEC_POLICY, zone_name);
@@ -179,16 +189,7 @@ int kdnssec_ctx_init(conf_t *conf, kdnssec_ctx_t *ctx, const knot_dname_t *zone_
 	conf_id_fix_default(&policy_id);
 	policy_load(ctx->policy, &policy_id);
 
-	conf_val_t keystore_id = conf_id_get(conf, C_POLICY, C_KEYSTORE, &policy_id);
-	conf_id_fix_default(&keystore_id);
-
-	conf_val_t val = conf_id_get(conf, C_KEYSTORE, C_BACKEND, &keystore_id);
-	unsigned backend = conf_opt(&val);
-
-	val = conf_id_get(conf, C_KEYSTORE, C_CONFIG, &keystore_id);
-	const char *config = conf_str(&val);
-
-	ret = keystore_load(config, backend, ctx->kasp_zone_path, &ctx->keystore);
+	ret = zone_init_keystore(conf, &policy_id, &ctx->keystore, NULL);
 	if (ret != KNOT_EOK) {
 		goto init_error;
 	}
@@ -207,7 +208,15 @@ int kdnssec_ctx_commit(kdnssec_ctx_t *ctx)
 		return KNOT_EINVAL;
 	}
 
-	// do something with keytore? Probably not..
+	if (ctx->policy->dnskey_ttl != UINT32_MAX &&
+	    ctx->policy->zone_maximal_ttl != UINT32_MAX) {
+		int ret = kasp_db_set_saved_ttls(ctx->kasp_db, ctx->zone->dname,
+		                                 ctx->policy->zone_maximal_ttl,
+		                                 ctx->policy->dnskey_ttl);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+	}
 
 	return kasp_zone_save(ctx->zone, ctx->zone->dname, ctx->kasp_db);
 }
@@ -232,3 +241,55 @@ void kdnssec_ctx_deinit(kdnssec_ctx_t *ctx)
 
 	memset(ctx, 0, sizeof(*ctx));
 }
+
+static void policy_from_zone(knot_kasp_policy_t *policy, const zone_contents_t *zone)
+{
+	// expects policy struct to be zeroed
+	policy->manual = true;
+	policy->single_type_signing = (node_rdataset(zone->apex, KNOT_RRTYPE_DNSKEY)->count == 1);
+	knot_rdataset_t *n3p = node_rdataset(zone->apex, KNOT_RRTYPE_NSEC3PARAM);
+	if (n3p != NULL) {
+		policy->nsec3_enabled = true;
+		policy->nsec3_iterations = knot_nsec3param_iters(n3p->rdata);
+		policy->nsec3_salt_length = knot_nsec3param_salt_len(n3p->rdata);
+	}
+	policy->signing_threads = 1;
+}
+
+int kdnssec_validation_ctx(conf_t *conf, kdnssec_ctx_t *ctx, const zone_contents_t *zone)
+{
+	if (ctx == NULL || zone == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	memset(ctx, 0, sizeof(*ctx));
+
+	ctx->zone = calloc(1, sizeof(*ctx->zone));
+	if (ctx->zone == NULL) {
+		return KNOT_ENOMEM;
+	}
+
+	ctx->policy = calloc(1, sizeof(*ctx->policy));
+	if (ctx->policy == NULL) {
+		free(ctx->zone);
+		return KNOT_ENOMEM;
+	}
+
+	policy_from_zone(ctx->policy, zone);
+	conf_val_t policy_id = conf_zone_get(conf, C_DNSSEC_POLICY, zone->apex->owner);
+	conf_val_t num_threads = conf_id_get(conf, C_POLICY, C_SIGNING_THREADS, &policy_id);
+	ctx->policy->signing_threads = conf_int(&num_threads);
+
+	int ret = kasp_zone_from_contents(ctx->zone, zone, ctx->policy->single_type_signing,
+	                                  ctx->policy->nsec3_enabled, &ctx->policy->nsec3_iterations,
+	                                  &ctx->keytag_conflict);
+	if (ret != KNOT_EOK) {
+		memset(ctx->zone, 0, sizeof(*ctx->zone));
+		kdnssec_ctx_deinit(ctx);
+		return ret;
+	}
+
+	ctx->now = knot_time();
+	ctx->validation_mode = true;
+	return KNOT_EOK;
+}
