@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,7 +30,8 @@
 #include "knot/zone/adjust.h"
 
 static int sign_init(zone_contents_t *zone, zone_sign_flags_t flags, zone_sign_roll_flags_t roll_flags,
-		     knot_lmdb_db_t *kaspdb, kdnssec_ctx_t *ctx, zone_sign_reschedule_t *reschedule)
+                     knot_time_t adjust_now, knot_lmdb_db_t *kaspdb, kdnssec_ctx_t *ctx,
+                     zone_sign_reschedule_t *reschedule)
 {
 	assert(zone);
 	assert(ctx);
@@ -41,6 +42,9 @@ static int sign_init(zone_contents_t *zone, zone_sign_flags_t flags, zone_sign_r
 	if (r != KNOT_EOK) {
 		return r;
 	}
+	if (adjust_now) {
+		ctx->now = adjust_now;
+	}
 
 	// perform nsec3resalt if pending
 
@@ -144,6 +148,7 @@ int knot_dnssec_nsec3resalt(kdnssec_ctx_t *ctx, knot_time_t *salt_changed, knot_
 int knot_dnssec_zone_sign(zone_update_t *update,
                           zone_sign_flags_t flags,
                           zone_sign_roll_flags_t roll_flags,
+                          knot_time_t adjust_now,
                           zone_sign_reschedule_t *reschedule)
 {
 	if (!update || !reschedule) {
@@ -157,7 +162,8 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 
 	// signing pipeline
 
-	result = sign_init(update->new_cont, flags, roll_flags, update->zone->kaspdb, &ctx, reschedule);
+	result = sign_init(update->new_cont, flags, roll_flags, adjust_now,
+	                   update->zone->kaspdb, &ctx, reschedule);
 	if (result != KNOT_EOK) {
 		log_zone_error(zone_name, "DNSSEC, failed to initialize (%s)",
 		               knot_strerror(result));
@@ -181,7 +187,8 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 		goto done;
 	}
 
-	result = zone_adjust_contents(update->new_cont, adjust_cb_flags, NULL, false, update->a_ctx->node_ptrs);
+	result = zone_adjust_contents(update->new_cont, adjust_cb_flags, NULL,
+	                              false, false, 1, update->a_ctx->node_ptrs);
 	if (result != KNOT_EOK) {
 		return result;
 	}
@@ -248,7 +255,7 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 	kdnssec_ctx_t ctx = { 0 };
 	zone_keyset_t keyset = { 0 };
 
-	result = sign_init(update->new_cont, 0, 0, update->zone->kaspdb, &ctx, reschedule);
+	result = sign_init(update->new_cont, 0, 0, 0, update->zone->kaspdb, &ctx, reschedule);
 	if (result != KNOT_EOK) {
 		log_zone_error(zone_name, "DNSSEC, failed to initialize (%s)",
 		               knot_strerror(result));
@@ -262,7 +269,8 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 		goto done;
 	}
 
-	result = zone_adjust_contents(update->new_cont, adjust_cb_flags, NULL, false, update->a_ctx->node_ptrs);
+	result = zone_adjust_contents(update->new_cont, adjust_cb_flags, NULL,
+	                              false, false, 1, update->a_ctx->node_ptrs);
 	if (result != KNOT_EOK) {
 		goto done;
 	}
@@ -322,3 +330,23 @@ knot_time_t knot_dnssec_failover_delay(const kdnssec_ctx_t *ctx)
 {
 	return ctx->now + ctx->policy->rrsig_prerefresh;
 }
+
+int knot_dnssec_validate_zone(zone_update_t *update, bool incremental)
+{
+	kdnssec_ctx_t ctx = { 0 };
+	int ret = kdnssec_validation_ctx(conf(), &ctx, update->new_cont);
+	if (ret == KNOT_EOK) {
+		ret = knot_zone_check_nsec_chain(update, &ctx, incremental);
+	}
+	if (ret == KNOT_EOK) {
+		knot_time_t unused = 0;
+		assert(ctx.validation_mode);
+		if (incremental) {
+			ret = knot_zone_sign_update(update, NULL, &ctx, &unused);
+		} else {
+			ret = knot_zone_sign(update, NULL, &ctx, &unused);
+		}
+	}
+	kdnssec_ctx_deinit(&ctx);
+	return ret;
+}
