@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -190,8 +190,7 @@ static void note_earliest_expiration(const knot_rdata_t *rrsig, knot_time_t *exp
 	*expires_at = knot_time_min(current, *expires_at);
 }
 
-// TODO: move somewhere?
-static bool rrsig_covers_type(const knot_rrset_t *rrsig, uint16_t type)
+bool rrsig_covers_type(const knot_rrset_t *rrsig, uint16_t type)
 {
 	if (knot_rrset_empty(rrsig)) {
 		return false;
@@ -295,6 +294,54 @@ static int add_missing_rrsigs(const knot_rrset_t *covered,
 	return result;
 }
 
+static bool key_used(bool ksk, bool zsk, uint16_t type,
+                     const knot_dname_t *owner, const knot_dname_t *zone_apex)
+{
+	if (knot_dname_cmp(owner, zone_apex) != 0) {
+		return zsk;
+	}
+	switch (type) {
+	case KNOT_RRTYPE_DNSKEY:
+	case KNOT_RRTYPE_CDNSKEY:
+	case KNOT_RRTYPE_CDS:
+		return ksk;
+	default:
+		return zsk;
+	}
+}
+
+/*!
+ * \brief Check that at least one correct signature exists to at least one DNSKEY.
+ *
+ * \param covered        RRSet bein validated.
+ * \param rrsigs         RRSIG with signatures.
+ * \param sign_ctx       Signing context (with keys == NULL)
+ * \param skip_crypto    Crypto operations might be skipped as they had been successful earlier.
+ *
+ * \return KNOT_E*
+ */
+static int validate_rrsigs(const knot_rrset_t *covered,
+                           const knot_rrset_t *rrsigs,
+                           zone_sign_ctx_t *sign_ctx,
+                           bool skip_crypto)
+{
+	for (size_t i = 0; i < sign_ctx->count; i++) {
+		const knot_kasp_key_t *key = &sign_ctx->dnssec_ctx->zone->keys[i];
+		if (!key_used(key->is_ksk, key->is_zsk, covered->type,
+		              covered->owner, sign_ctx->dnssec_ctx->zone->dname)) {
+			continue;
+		}
+
+		uint16_t valid_at;
+		if (valid_signature_exists(covered, rrsigs, key->key, sign_ctx->sign_ctxs[i],
+		                           sign_ctx->dnssec_ctx, skip_crypto, &valid_at)) {
+			return KNOT_EOK;
+		}
+	}
+
+	return KNOT_DNSSEC_ENOSIG;
+}
+
 /*!
  * \brief Add all RRSIGs into the changeset for removal.
  *
@@ -423,7 +470,8 @@ static int remove_standalone_rrsigs(const zone_node_t *node,
 static int sign_node_rrsets(const zone_node_t *node,
                             zone_sign_ctx_t *sign_ctx,
                             changeset_t *changeset,
-                            knot_time_t *expires_at)
+                            knot_time_t *expires_at,
+                            dnssec_validation_hint_t *hint)
 {
 	assert(node);
 	assert(sign_ctx);
@@ -435,12 +483,22 @@ static int sign_node_rrsets(const zone_node_t *node,
 
 	for (int i = 0; result == KNOT_EOK && i < node->rrset_count; i++) {
 		knot_rrset_t rrset = node_rrset_at(node, i);
+		assert(rrset.type != KNOT_RRTYPE_ANY);
+
 		if (!knot_zone_sign_rr_should_be_signed(node, &rrset)) {
-			result = remove_rrset_rrsigs(rrset.owner, rrset.type, &rrsigs, changeset);
+			if (!sign_ctx->dnssec_ctx->validation_mode) {
+				result = remove_rrset_rrsigs(rrset.owner, rrset.type, &rrsigs, changeset);
+			}
 			continue;
 		}
 
-		if (sign_ctx->dnssec_ctx->rrsig_drop_existing) {
+		if (sign_ctx->dnssec_ctx->validation_mode) {
+			result = validate_rrsigs(&rrset, &rrsigs, sign_ctx, skip_crypto);
+			if (result != KNOT_EOK) {
+				hint->node = node->owner;
+				hint->rrtype = rrset.type;
+			}
+		} else if (sign_ctx->dnssec_ctx->rrsig_drop_existing) {
 			result = force_resign_rrset(&rrset, &rrsigs,
 			                            sign_ctx, changeset);
 		} else {
@@ -463,6 +521,7 @@ typedef struct {
 	zone_sign_ctx_t *sign_ctx;
 	changeset_t changeset;
 	knot_time_t expires_at;
+	dnssec_validation_hint_t *hint;
 	size_t num_threads;
 	size_t thread_index;
 	size_t rrset_index;
@@ -493,7 +552,8 @@ static int sign_node(zone_node_t *node, void *data)
 	}
 
 	int result = sign_node_rrsets(node, args->sign_ctx,
-	                              &args->changeset, &args->expires_at);
+	                              &args->changeset, &args->expires_at,
+	                              args->hint);
 
 	return result;
 }
@@ -531,9 +591,9 @@ static int zone_tree_sign(zone_tree_t *tree,
                           zone_update_t *update,
                           knot_time_t *expires_at)
 {
-	assert(zone_keys);
+	assert(zone_keys || dnssec_ctx->validation_mode);
 	assert(dnssec_ctx);
-	assert(update);
+	assert(update || dnssec_ctx->validation_mode);
 
 	int ret = KNOT_EOK;
 	node_sign_args_t args[num_threads];
@@ -543,16 +603,19 @@ static int zone_tree_sign(zone_tree_t *tree,
 	// init context structures
 	for (size_t i = 0; i < num_threads; i++) {
 		args[i].tree = tree;
-		args[i].sign_ctx = zone_sign_ctx(zone_keys, dnssec_ctx);
+		args[i].sign_ctx = dnssec_ctx->validation_mode
+		                 ? zone_validation_ctx(dnssec_ctx)
+		                 : zone_sign_ctx(zone_keys, dnssec_ctx);
 		if (args[i].sign_ctx == NULL) {
 			ret = KNOT_ENOMEM;
 			break;
 		}
-		ret = changeset_init(&args[i].changeset, update->zone->name);
+		ret = changeset_init(&args[i].changeset, dnssec_ctx->zone->dname);
 		if (ret != KNOT_EOK) {
 			break;
 		}
 		args[i].expires_at = 0;
+		args[i].hint = &update->validation_hint;
 		args[i].num_threads = num_threads;
 		args[i].thread_index = i;
 		args[i].rrset_index = 0;
@@ -586,16 +649,19 @@ static int zone_tree_sign(zone_tree_t *tree,
 	}
 
 	// collect return code and results
-	for (size_t i = 0; i < num_threads && ret == KNOT_EOK; i++) {
-		if (args[i].thread_init_errcode != 0) {
-			ret = knot_map_errno_code(args[i].thread_init_errcode);
-		} else {
-			ret = args[i].errcode;
-			if (ret == KNOT_EOK) {
-				ret = zone_update_apply_changeset(update, &args[i].changeset); // _fix not needed
-				*expires_at = knot_time_min(*expires_at, args[i].expires_at);
+	for (size_t i = 0; i < num_threads; i++) {
+		if (ret == KNOT_EOK) {
+			if (args[i].thread_init_errcode != 0) {
+				ret = knot_map_errno_code(args[i].thread_init_errcode);
+			} else {
+				ret = args[i].errcode;
+				if (ret == KNOT_EOK && !dnssec_ctx->validation_mode) {
+					ret = zone_update_apply_changeset(update, &args[i].changeset); // _fix not needed
+					*expires_at = knot_time_min(*expires_at, args[i].expires_at);
+				}
 			}
 		}
+		assert(!dnssec_ctx->validation_mode || changeset_empty(&args[i].changeset));
 		changeset_clear(&args[i].changeset);
 		zone_sign_ctx_free(args[i].sign_ctx);
 	}
@@ -650,8 +716,9 @@ int knot_zone_sign(zone_update_t *update,
                    const kdnssec_ctx_t *dnssec_ctx,
                    knot_time_t *expire_at)
 {
-	if (!update || !zone_keys || !dnssec_ctx || !expire_at ||
-	    dnssec_ctx->policy->signing_threads < 1) {
+	if (!update || !dnssec_ctx || !expire_at ||
+	    dnssec_ctx->policy->signing_threads < 1 ||
+	    (zone_keys == NULL && !dnssec_ctx->validation_mode)) {
 		return KNOT_EINVAL;
 	}
 
@@ -963,8 +1030,9 @@ int knot_zone_sign_update(zone_update_t *update,
                           const kdnssec_ctx_t *dnssec_ctx,
                           knot_time_t *expire_at)
 {
-	if (update == NULL || zone_keys == NULL || dnssec_ctx == NULL || expire_at == NULL ||
-	    dnssec_ctx->policy->signing_threads < 1) {
+	if (update == NULL || dnssec_ctx == NULL || expire_at == NULL ||
+	    dnssec_ctx->policy->signing_threads < 1 ||
+	    (zone_keys == NULL && !dnssec_ctx->validation_mode)) {
 		return KNOT_EINVAL;
 	}
 
@@ -981,6 +1049,13 @@ int knot_zone_sign_update(zone_update_t *update,
 		if (ret == KNOT_EOK) {
 			ret = zone_tree_apply(update->a_ctx->node_ptrs, set_signed, NULL);
 		}
+		if (ret == KNOT_EOK && dnssec_ctx->validation_mode) {
+			ret = zone_tree_sign(update->a_ctx->nsec3_ptrs, dnssec_ctx->policy->signing_threads,
+			                     zone_keys, dnssec_ctx, update, expire_at);
+		}
+		if (ret == KNOT_EOK && dnssec_ctx->validation_mode) {
+			ret = zone_tree_apply(update->a_ctx->nsec3_ptrs, set_signed, NULL);
+		}
 	}
 
 	return ret;
