@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,53 +16,58 @@
 
 #pragma once
 
-#include "sys/socket.h"
-
 #include "knot/conf/conf.h"
 #include "knot/common/evsched.h"
 #include "knot/common/fdset.h"
 #include "knot/journal/knot_lmdb.h"
 #include "knot/server/dthreads.h"
 #include "knot/worker/pool.h"
+#include "knot/zone/catalog.h"
 #include "knot/zone/zonedb.h"
-#include "contrib/ucw/lists.h"
 
-/* Forwad declarations. */
 struct server;
+struct knot_xdp_socket;
 
-/*! \brief I/O handler structure.
-  */
-typedef struct iohandler {
-	struct node        n;
-	struct server      *server; /*!< Reference to server */
-	dt_unit_t          *unit;   /*!< Threading unit */
-	unsigned           *thread_state; /*!< Thread state */
-	unsigned           *thread_id; /*!< Thread identifier. */
+/*!
+ * \brief I/O handler structure.
+ */
+typedef struct {
+	struct server *server;  /*!< Reference to server. */
+	dt_unit_t *unit;        /*!< Threading unit. */
+	unsigned *thread_state; /*!< Thread states. */
+	unsigned *thread_id;    /*!< Thread identifiers per all handlers. */
 } iohandler_t;
 
-/*! \brief Server state flags.
+/*!
+ * \brief Server state flags.
  */
 typedef enum {
 	ServerIdle    = 0 << 0, /*!< Server is idle. */
 	ServerRunning = 1 << 0, /*!< Server is running. */
-} server_state;
+} server_state_t;
 
 /*!
  * \brief Server interface structure.
  */
-typedef struct iface {
-	struct node n;
+typedef struct {
 	int *fd_udp;
-	int fd_udp_count;
+	unsigned fd_udp_count;
 	int *fd_tcp;
-	int fd_tcp_count;
+	unsigned fd_tcp_count;
+	int *fd_xdp;
+	unsigned fd_xdp_count;
+	unsigned xdp_first_thread_id;
+	struct knot_xdp_socket **xdp_sockets;
 	struct sockaddr_storage addr;
 } iface_t;
 
-/* Handler indexes. */
+/*!
+ * \brief Handler indexes.
+ */
 enum {
 	IO_UDP = 0,
-	IO_TCP = 1
+	IO_TCP = 1,
+	IO_XDP = 2,
 };
 
 /*!
@@ -71,21 +76,20 @@ enum {
  * Keeps references to all important structures needed for operation.
  */
 typedef struct server {
-
 	/*! \brief Server state tracking. */
 	volatile unsigned state;
 
-	/*! \brief Zone database. */
 	knot_zonedb_t *zone_db;
 	knot_lmdb_db_t timerdb;
 	knot_lmdb_db_t journaldb;
 	knot_lmdb_db_t kaspdb;
+	catalog_t catalog;
 
 	/*! \brief I/O handlers. */
 	struct {
 		unsigned size;
 		iohandler_t handler;
-	} handlers[2];
+	} handlers[3];
 
 	/*! \brief Background jobs. */
 	worker_pool_t *workers;
@@ -94,8 +98,11 @@ typedef struct server {
 	evsched_t sched;
 
 	/*! \brief List of interfaces. */
-	list_t *ifaces;
+	iface_t *ifaces;
+	size_t n_ifaces;
 
+	/*! \brief Pending changes to catalog member zones. */
+	catalog_update_t catalog_upd;
 } server_t;
 
 /*!
