@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "libknot/libknot.h"
 #include "knot/zone/semantic-check.h"
 #include "knot/dnssec/rrset-sign.h"
+#include "knot/dnssec/zone-keys.h"
 #include "knot/dnssec/zone-nsec.h"
 
 static const char *error_messages[SEM_ERR_UNKNOWN + 1] = {
@@ -193,40 +194,6 @@ static const struct check_function CHECK_FUNCTIONS[] = {
 static const int CHECK_FUNCTIONS_LEN = sizeof(CHECK_FUNCTIONS)
                                      / sizeof(struct check_function);
 
-static int dnssec_key_from_rdata(dnssec_key_t **key, const knot_dname_t *owner,
-				 const uint8_t *rdata, size_t rdlen)
-{
-	if (!key || !rdata || rdlen == 0) {
-		return KNOT_EINVAL;
-	}
-
-	const dnssec_binary_t binary_key = {
-		.size = rdlen,
-		.data = (uint8_t *)rdata
-	};
-
-	dnssec_key_t *new_key = NULL;
-	int ret = dnssec_key_new(&new_key);
-	if (ret != DNSSEC_EOK) {
-		return KNOT_ENOMEM;
-	}
-	ret = dnssec_key_set_rdata(new_key, &binary_key);
-	if (ret != DNSSEC_EOK) {
-		dnssec_key_free(new_key);
-		return KNOT_ENOMEM;
-	}
-	if (owner) {
-		ret = dnssec_key_set_dname(new_key, owner);
-		if (ret != DNSSEC_EOK) {
-			dnssec_key_free(new_key);
-			return KNOT_ENOMEM;
-		}
-	}
-
-	*key = new_key;
-	return KNOT_EOK;
-}
-
 static int check_signature(const knot_rdata_t *rrsig, const dnssec_key_t *key,
                            const knot_rrset_t *covered)
 {
@@ -256,7 +223,7 @@ static int check_signature(const knot_rdata_t *rrsig, const dnssec_key_t *key,
 		goto fail;
 	}
 
-	if (dnssec_sign_verify(sign_ctx, &signature) != KNOT_EOK) {
+	if (dnssec_sign_verify(sign_ctx, false, &signature) != KNOT_EOK) {
 		ret = KNOT_EINVAL;
 		goto fail;
 	}
@@ -1025,9 +992,9 @@ static int check_nsec3(const zone_node_t *node, semchecks_data_t *data)
 	                                                              next_dname);
 	if (next_nsec3 == NULL || node_prev(next_nsec3) != nsec3_node) {
 		uint8_t *next = NULL;
-		int32_t next_len = base32hex_encode_alloc(next_dname_str,
-		                                          next_dname_str_size,
-		                                          &next);
+		int32_t next_len = knot_base32hex_encode_alloc(next_dname_str,
+		                                               next_dname_str_size,
+		                                               &next);
 		char *hash_info = NULL;
 		if (next != NULL) {
 			hash_info = sprintf_alloc("(next hash %.*s)", next_len, next);
