@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,12 +22,14 @@
 #include "knot/journal/journal_basic.h"
 #include "knot/events/events.h"
 #include "knot/updates/changesets.h"
+#include "knot/zone/catalog.h"
 #include "knot/zone/contents.h"
 #include "knot/zone/timers.h"
 #include "libknot/dname.h"
 #include "libknot/packet/pkt.h"
 
 struct zone_update;
+struct zone_backup_ctx;
 
 /*!
  * \brief Zone flags.
@@ -38,6 +40,8 @@ typedef enum zone_flag_t {
 	ZONE_FORCE_FLUSH    = 1 << 2, /*!< Force zone flush. */
 	ZONE_FORCE_KSK_ROLL = 1 << 3, /*!< Force KSK/CSK rollover. */
 	ZONE_FORCE_ZSK_ROLL = 1 << 4, /*!< Force ZSK rollover. */
+	ZONE_IS_CATALOG     = 1 << 5, /*!< This is a catalog. */
+	ZONE_IS_CAT_MEMBER  = 1 << 6, /*!< This zone exists according to a catalog. */
 } zone_flag_t;
 
 /*!
@@ -82,6 +86,13 @@ typedef struct zone
 	/*! \brief Ptr to journal DB (in struct server) */
 	knot_lmdb_db_t *kaspdb;
 
+	/*! \brief Zone backup context (NULL unless backup pending). */
+	struct zone_backup_ctx *backup_ctx;
+
+	/*! \brief Ptr to catalog and ist changeset changes (in struct server) */
+	catalog_t *catalog;
+	catalog_update_t *catalog_upd;
+
 	/*! \brief Preferred master lock. */
 	pthread_mutex_t preferred_lock;
 	/*! \brief Preferred master for remote operation. */
@@ -110,6 +121,14 @@ zone_t* zone_new(const knot_dname_t *name);
  */
 void zone_free(zone_t **zone_ptr);
 
+/*!
+ * \brief Clear zone contents (->SERVFAIL), reset modules, plan LOAD.
+ *
+ * \param conf   Current configuration.
+ * \param zone   Zone to be re-set.
+ */
+void zone_reset(conf_t *conf, zone_t *zone);
+
 /*!
  * \brief Clears possible control update transaction.
  *
