@@ -57,6 +57,8 @@ enum knot_error {
 	KNOT_ERANGE        = -ERANGE,
 	KNOT_EADDRNOTAVAIL = -EADDRNOTAVAIL,
 
+	KNOT_ERRNO_ERROR   = -500,
+
 	KNOT_ERROR_MIN = -1000,
 
 	/* General errors. */
@@ -92,9 +94,16 @@ enum knot_error {
 	KNOT_ESOAINVAL,
 	KNOT_ETRAIL,
 	KNOT_EPROCESSING,
+	KNOT_EPROGRESS,
+	KNOT_ELOOP,
+	KNOT_EPROGRAM,
+	KNOT_EFD,
+
+	KNOT_GENERAL_ERROR = -900,
 
 	/* Control states. */
 	KNOT_CTL_ESTOP,
+	KNOT_CTL_EZONE,
 
 	/* Network errors. */
 	KNOT_NET_EADDR,
@@ -119,6 +128,10 @@ enum knot_error {
 	/* DNSSEC errors. */
 	KNOT_DNSSEC_EMISSINGKEYTYPE,
 	KNOT_DNSSEC_ENOKEY,
+	KNOT_DNSSEC_ENOSIG,
+	KNOT_DNSSEC_ENSEC_BITMAP,
+	KNOT_DNSSEC_ENSEC_CHAIN,
+	KNOT_DNSSEC_ENSEC3_OPTOUT,
 
 	/* Yparser errors. */
 	KNOT_YP_ECHAR_TAB,
@@ -149,11 +162,6 @@ enum knot_error {
 	KNOT_INVALID_KEY_NAME,
 	KNOT_NO_PUBLIC_KEY,
 	KNOT_NO_PRIVATE_KEY,
-
-	/* Another general errors. */
-	KNOT_ELOOP,
-
-	/* Backported. */
 	KNOT_NO_READY_KEY,
 
 	KNOT_ERROR_MAX = -501
@@ -162,11 +170,12 @@ enum knot_error {
 /*!
  * \brief Map POSIX errno code to Knot error code.
  *
- * \param code Errno code to transform (set -1 to use the current errno).
+ * \param code       Errno code to transform (set -1 to use the current errno).
+ * \param dflt_error Default return value, if code is unknown.
  *
- * \return Mapped errno or KNOT_ERROR if unknown.
+ * \return Mapped errno or the value of dflt_error if unknown.
  */
-inline static int knot_map_errno_code(int code)
+inline static int knot_map_errno_code_def(int code, int dflt_error)
 {
 	if (code < 0) {
 		code = errno;
@@ -204,8 +213,8 @@ inline static int knot_map_errno_code(int code)
 		ERR_ITEM(ERANGE),
 		ERR_ITEM(EADDRNOTAVAIL),
 
-		/* Terminator - default value. */
-		{ 0, KNOT_ERROR }
+		/* Terminator - the value isn't used. */
+		{ 0, KNOT_ERRNO_ERROR }
 	};
 	#undef ERR_ITEM
 
@@ -215,17 +224,41 @@ inline static int knot_map_errno_code(int code)
 		err++;
 	}
 
-	return err->libknot_code;
+	return err->errno_code != 0 ? err->libknot_code : dflt_error;
+}
+
+/*!
+ * \brief Map POSIX errno code to Knot error code.
+ *
+ * \param code Errno code to transform (set -1 to use the current errno).
+ *
+ * \return Mapped errno or KNOT_ERRNO_ERROR if unknown.
+ */
+inline static int knot_map_errno_code(int code)
+{
+	return knot_map_errno_code_def(code, KNOT_ERRNO_ERROR);
 }
 
 /*!
  * \brief Get a POSIX errno mapped to Knot error code.
  *
- * \return Mapped errno or KNOT_ERROR if unknown.
+ * \param dflt_error Default return value, if code is unknown.
+ *
+ * \return Mapped errno.
+ */
+inline static int knot_map_errno_def(int dflt_error)
+{
+	return knot_map_errno_code_def(-1, dflt_error);
+}
+
+/*!
+ * \brief Get a POSIX errno mapped to Knot error code.
+ *
+ * \return Mapped errno or KNOT_ERRNO_ERROR if unknown.
  */
 inline static int knot_map_errno(void)
 {
-	return knot_map_errno_code(-1);
+	return knot_map_errno_code_def(-1, KNOT_ERRNO_ERROR);
 }
 
 /*! @} */
