@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,6 +58,8 @@ static const struct error errors[] = {
 	{ KNOT_ERANGE,        "value is out of range" },
 	{ KNOT_EADDRNOTAVAIL, "address is not available" },
 
+	{ KNOT_ERRNO_ERROR,   "unknown system error" },
+
 	/* General errors. */
 	{ KNOT_ERROR,        "failed" },
 	{ KNOT_EPARSEFAIL,   "parser failed" },
@@ -83,7 +85,7 @@ static const struct error errors[] = {
 	{ KNOT_ETIMEOUT,     "connection timeout" },
 	{ KNOT_ENODIFF,      "cannot create zone diff" },
 	{ KNOT_ENOTSIG,      "expected a TSIG or SIG(0)" },
-	{ KNOT_ELIMIT,       "exceeded response rate limit" },
+	{ KNOT_ELIMIT,       "exceeded limit" },
 	{ KNOT_EZONESIZE,    "zone size exceeded" },
 	{ KNOT_EOF,          "end of file" },
 	{ KNOT_ESYSTEM,      "system error" },
@@ -91,9 +93,16 @@ static const struct error errors[] = {
 	{ KNOT_ESOAINVAL,    "SOA mismatch" },
 	{ KNOT_ETRAIL,       "trailing data" },
 	{ KNOT_EPROCESSING,  "processing error" },
+	{ KNOT_EPROGRESS,    "in progress" },
+	{ KNOT_ELOOP,        "loop detected" },
+	{ KNOT_EPROGRAM,     "program not loaded" },
+	{ KNOT_EFD,          "file descriptor error" },
+
+	{ KNOT_GENERAL_ERROR, "unknown general error" },
 
 	/* Control states. */
 	{ KNOT_CTL_ESTOP,     "stopping server" },
+	{ KNOT_CTL_EZONE,     "operation failed for some zones" },
 
 	/* Network errors. */
 	{ KNOT_NET_EADDR,    "bad address or host name" },
@@ -118,6 +127,10 @@ static const struct error errors[] = {
 	/* DNSSEC errors. */
 	{ KNOT_DNSSEC_ENOKEY,          "no keys for signing" },
 	{ KNOT_DNSSEC_EMISSINGKEYTYPE, "missing active KSK or ZSK" },
+	{ KNOT_DNSSEC_ENOSIG,          "no valid signature for a record" },
+	{ KNOT_DNSSEC_ENSEC_BITMAP,    "missing NSEC(3) record or wrong bitmap" },
+	{ KNOT_DNSSEC_ENSEC_CHAIN,     "inconsistent NSEC(3) chain" },
+	{ KNOT_DNSSEC_ENSEC3_OPTOUT,   "wrong NSEC3 opt-out" },
 
 	/* Yparser errors. */
 	{ KNOT_YP_ECHAR_TAB,     "tabulator character is not allowed" },
@@ -148,14 +161,10 @@ static const struct error errors[] = {
 	{ KNOT_INVALID_KEY_NAME,      "invalid key name" },
 	{ KNOT_NO_PUBLIC_KEY,         "no public key" },
 	{ KNOT_NO_PRIVATE_KEY,        "no private key" },
-
-	/* Another general errors. */
-	{ KNOT_ELOOP,                 "loop detected" },
-
-	/* Backported. */
 	{ KNOT_NO_READY_KEY,          "no key ready for submission" },
 
-	{ KNOT_ERROR, NULL } /* Terminator */
+	/* Terminator */
+	{ KNOT_ERROR, NULL }
 };
 
 /*!
